package soundtest;

import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.analysis.interpolation.SplineInterpolator;

public class DownsampleSoundTransformation implements SoundTransformation {

	private int percent = 20;
	
	public DownsampleSoundTransformation (int percent) {
		this.percent = percent;
    }


	@Override
	public Sound transform (Sound input) {
		Sound fs = DownsampleSoundTransformation.precisionFilter (
				DownsampleSoundTransformation.normalize (input), this.percent);
		UnivariateFunction function = this.getFunction (fs);
		return DownsampleSoundTransformation.buildSoundFromFunction (function,
		        (int) fs.getTime () [fs.getTime ().length - 1]);
	}

	protected UnivariateFunction getFunction (Sound fs) {
		return new SplineInterpolator ()
		        .interpolate (fs.getTime (), fs.getSamples ());
    }


	private static Sound buildSoundFromFunction (UnivariateFunction function,
	        int length) {
		double [] result = new double [length];
		for (int i = 0; i < length; i++) {
			result [i] = function.value (i);
		}
		return new Sound (result);
	}

	private static Sound normalize (Sound sound) {
		double [] data = sound.getSamples ();
		double [] newdata = new double [sound.getSamples ().length];
		// this is the raw audio data -- no header

		// find the max:
		double max = 0;
		for (int i = 0; i < data.length; i++) {
			if (Math.abs (data [i]) > max)
				max = Math.abs (data [i]);
		}

		// now find the result, with scaling:
		for (int i = 0; i < data.length; i++) {
			newdata [i] = Math.round (Short.MAX_VALUE * (data [i]) / max);
		}
		// normalized result in newdata
		return new Sound (newdata);
	}

	private static Sound precisionFilter (Sound sound, float percent) {
		float total = 100;
		if (percent == total){
			return new Sound (sound.getSamples ());
		}
		float nbSamples = sound.getSamples ().length;
		float nbFiltered = percent / total * nbSamples;
		float incr = nbSamples / nbFiltered;
		double [] data = sound.getSamples ();
		double [] ret = new double [(int) (nbFiltered)];
		double [] time = new double [(int) (nbFiltered)];
		for (float i = 0; i < incr * nbFiltered; i += incr) {
			int j = (int) (i / incr);
			if (j < ret.length) {
				ret [j] = data [(int) i];
				time [j] = i;
			}
		}
		return new FilteredSound (ret, time);
	}
}
