package soundtest;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFileFormat;
import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

public class TransformSound {

	TransformObserver[] observers = new TransformObserver [0];
	
	public TransformSound (TransformObserver... observers) {
		this.observers = observers;
    }

	public TransformSound () {
    }

	public AudioInputStream transformAudioStream (AudioInputStream ais, SoundTransformation st ) {
		Sound [] s = this.fromAudioInputStream (ais);
		Sound [] ss = new Sound [s.length];
		for (int i = 0; i < s.length; i++) {
			this.notifyAll ("Transforming channel n°" + (i + 1) + "/" + s.length);
			ss [i] = st.transform (s [i]);
		}
		this.notifyAll ("Transform done");
		return this.toStream (ss, ais.getFormat ());

	}

	private void notifyAll (String s){
		for (TransformObserver to : this.observers){
			to.notify (s);
		}
	}
	
	private Sound [] fromAudioInputStream (AudioInputStream ais) {
		this.notifyAll ("Converting input into java object");
		AudioInfo ai = new AudioInfo (ais);
		Sound [] ret = new Sound [ai.getNumberOfChannels ()];
		for (int t = 0; t < ai.getNumberOfChannels (); t++) {
			int [] audio = ai.getAudio (t);
			int audioLength = audio.length;
			double [] val = new double [audioLength];
			for (int i = 0; i < audio.length; i++) {
				val [i] = audio [i];
			}
			ret [t] = new Sound (val);
			ret [t].setAudio (audio);
		}
		this.notifyAll ("Convert done");
		return ret;
	}

	private AudioInputStream toStream (Sound [] channels, AudioFormat audioFormat) {

		byte [] data = new byte [2 * channels.length
		        * channels [0].getSamples ().length];
		int i = 0;
		for (int j = 0; j < channels.length; j++) {
			double [] input = channels [j].getSamples ();
			int k = 0;
			for (; i < input.length; i++,k++) {
				int val = (int)input [k];
				byte part1 = (byte) val;
				byte part2 = (byte) (val >> 8);
				byte part3 = (byte) (val >> 16);
				byte part4 = (byte) (val >> 24);
				data [4 * i + 0] = part1;
				data [4 * i + 1] = part2;
				data [4 * i + 2] = part3;
				data [4 * i + 3] = part4;
			}
		}

		this.notifyAll ("Creating output file");
		// now save the file
		ByteArrayInputStream bais = new ByteArrayInputStream (data);
		return new AudioInputStream (bais, audioFormat,  2 * channels.length * channels [0].getSamples ().length);
	}

	public void transformWav (File fOrigin, File fDest, SoundTransformation st) throws UnsupportedAudioFileException, IOException {
			File file = fOrigin;
			AudioInputStream ais1 = AudioSystem.getAudioInputStream (file);
			this.notifyAll ("input : " + ais1.getFormat ().toString ());
			AudioInputStream ais2 = this.transformAudioStream (ais1, st);
			ais1 = AudioSystem.getAudioInputStream (file);
			AudioSystem.write (ais2, AudioFileFormat.Type.WAVE, fDest);
			this.notifyAll ("Wrote output");
			this.notifyAll ("output : " + ais2.getFormat ().toString ());
	}
}
